%tesa_interpdata()     - replaces removed data using weighted forward
%                        and backward prediction. This is achieved by
%                        finding k nearest neighbors of target-points in
%                        segments of specific durations prior to and afer
%                        the gap.
%Usage:
%   >>  EEG = pop_tesa_fillGaps( EEG ); % Pop up window
%   >>  EEG = pop_tesa_fillGaps( EEG,pulse,cor,time_samples,memb,nnei);
%
% Inputs:
%   EEG                 - EEGLAB EEG structure
%*Optional(in set order)
%   pulse               - string describing type of pulse, either
%                         'Sinlge Pulse' or 'Double Pulse'. If not given as
%                         an input the user is asked to clarify
%   cor                 - flag for extra baseline correction, (default 1: excecute)
%   time_samples        - two point array with the duration of the prediction segments                      
%   memb                - number of embending dimensions (default 50)   
%   nnei                - number of nearest neighbors to be detectd (default 2)
%
% Outputs:
%   EEG                 - EEGLAB EEG structure

function [EEG] = tesa_fillGaps(EEG,pulse,cor,time_samples,memb,nnei)
%initialize default options
if nargin<2
    pulse = questdlg('Single or Paired pusle EEG data','TMS Pulse','Single Pulse','Paired Pulse','Single Pulse');
	cor = 1;
    time_samples = [-EEG.xmin*1000 EEG.xmax*1000]; %set duration as whole epoch
    memb = 50;
    nnei = 2;
elseif nargin<3
	cor = 1;
    time_samples = [-EEG.xmin*1000 EEG.xmax*1000];
    memb = 50;
    nnei = 2;
elseif nargin<4
	time_samples = [-EEG.xmin*1000 EEG.xmax*1000];
    memb = 50;
    nnei = 2;
elseif nargin<5
	memb = 50;
    nnei = 2;
elseif nargin<6
	nnei = 2;
end

z = size(EEG.tmscut,2); % last input in tmscut field

% from pop-up option: set actual durations 
if isempty(time_samples)
    time_samples = [-EEG.xmin*1000 EEG.xmax*1000];
end

sbeforerec = time_samples(1,1)*EEG.srate/1000;
safterrec = time_samples(1,2)*EEG.srate/1000;

if strcmp(pulse,'Single Pulse')
    % Single pulse
    %check that prediciton segments' duration does not exceed data range
    if -sbeforerec < EEG.times(1,1) || safterrec > EEG.times(1,end)
        error('Time duration of learning sets are out of data range. Note that time value is in ms.');
    end
    %check to see that TMS data has been removed
    if ~isfield(EEG,'tmscut')
        error('Artifact data not removed. Please run pop_tesa_removedata first.');
    end
    
    %Check that tmscut fields generated by tesa_removedata.m contain data
    if isfield(EEG,'tmscut')
        if isempty(EEG.tmscut(1).tmscut1) || isempty(EEG.tmscut(1).tmscut2)
            error('Artifact removal time points do not contain data. Please re-run pop_tesa_removedata.');
        end
    end
    % retrieve indexes corresponding to start and end of the TMS gap
    if isequal(EEG.srate,EEG.tmscut(z).srate)
        sbeforeTMS = EEG.tmscut(z).tmscut1;
        safterTMS = EEG.tmscut(z).tmscut2;
    elseif ~isequal(EEG.srate,EEG.tmscut(z).srate)
        [~,sbeforeTMS] = min(abs(EEG.times-EEG.tmscut(z).cutTimesTMS(1,1)));
        [~,safterTMS] = min(abs(EEG.times-EEG.tmscut(z).cutTimesTMS(1,2)));
    end
    %index of the TMS event
    imin = floor(EEG.event(1).latency);
	%extra baseline correction
	if cor == 1
		dur = EEG.pnts/4;
		npreTMS = dur*EEG.srate/1000;
		npostTMS = npreTMS;
		EEG = baselineCorForGaps(EEG,npreTMS,npostTMS,sbeforeTMS,safterTMS,imin);
	end
	%fill gaps
    EEG = fillGaps(EEG,sbeforeTMS,safterTMS,sbeforerec,safterrec,imin,memb,nnei);
    %add new indexes and information in tmscut
    EEG.tmscut(z+1).cutTimesTMS = EEG.tmscut(z).cutTimesTMS;
    EEG.tmscut(z+1).replaceTimes = EEG.tmscut(z+1).cutTimesTMS;
    EEG.tmscut(z+1).replacement = EEG.data(:,sbeforeTMS:safterTMS,:);
    EEG.tmscut(z+1).tmscut1 = sbeforeTMS;
    EEG.tmscut(z+1).tmscut2 = safterTMS;
    EEG.tmscut(z+1).tmscut3 = [];
    EEG.tmscut(z+1).tmscut3 = [];
    EEG.tmscut(z+1).srate = EEG.srate;
    EEG.tmscut(z+1).interpolated = 'applied prediction';
else
    %Paired Pulse
    %check that the segment durations do no exceed data range
    if -sbeforerec < EEG.times(1,1) || safterrec > EEG.times(1,end)
        error('Time duration of learning sets are out of data range. Note that time value is in ms.');
    end
    
    if ~isfield(EEG,'tmscut')
        error('Artifact data not removed. Please run pop_tesa_removedata first.');
    end
    
    %Check that tmscut fields generated by tesa_removedata.m contain data
    if isfield(EEG,'tmscut')
        if isempty(EEG.tmscut(1).tmscut1) || isempty(EEG.tmscut(1).tmscut2)
            error('Artifact removal time points do not contain data. Please re-run pop_tesa_removedata.');
        end
    end
    % retrieve indexes corresponding to start and end of both pulses
    if isequal(EEG.srate,EEG.tmscut(z).srate)
        sbeforeTMS1 = EEG.tmscut(z).tmscut1;
        safterTMS1 = EEG.tmscut(z).tmscut2;
        sbeforeTMS2 = EEG.tmscut(z).tmscut3;
        safterTMS2 = EEG.tmscut(z).tmscut4;
    elseif ~isequal(EEG.srate,EEG.tmscut(z).srate)
        sbeforeTMS1 = EEG.tmscut(z).tmscut1*EEG.srate/EEG.tmscut(z).srate;
        safterTMS1 = EEG.tmscut(z).tmscut2*EEG.srate/EEG.tmscut(z).srate;
        sbeforeTMS2 = EEG.tmscut(z).tmscut3*EEG.srate/EEG.tmscut(z).srate;
        safterTMS2 = EEG.tmscut(z).tmscut4*EEG.srate/EEG.tmscut(z).srate;
    end
    %indexes of TMS event of both pulses
    imin1 = sbeforeTMS1 + abs(EEG.tmscut(z).cutTimesTMS(1,1));
    imin2 = sbeforeTMS2 + abs(EEG.tmscut(z).cutTimesTMS(1,1));
    
    %Paired pulse - If the durations are left as initialized:
    %For the first pulse :  The duration of the forward prediction segment
    %is set form the beginning of the epoch until the beginning of the gap.
    %The duration of the backward prediction is set from the end of the
    %first gap until the beginning of second gap.
    %For the second pulse : The duration of the forward prediction segment
    %is set as the interval between the two gaps and the backward
    %prediction segment is set from the end of the second gap to the end of
    %the epoch.
     if -sbeforerec == EEG.times(1,1)
        sbeforerec1 = abs(EEG.times(1,1)) - abs(EEG.times(1,imin1));
        sbeforerec2 = abs(EEG.times(1,safterTMS1))+EEG.times(1,imin2);
    else
        sbeforerec1 = sebeforerec;
        sbeforerec2 = sbeforerec;
    end
    if safterrec == EEG.times(1,end)
        safterrec1 = abs(EEG.times(1,imin1))+EEG.times(1,EEG.TMS_period2remove_b(1,1));
        safterrec2 = abs(EEG.times(1,end)) - abs(EEG.times(1,imin2))-1;
    else
        safterrec1 = safterrec;
        safterrec2 = safterrec;
    end
    %check that new durations do not overlap with gaps
    if EEG.times(imin1) - sbeforerec1< EEG.times(1,1) || EEG.times(imin2) + safterrec2 > EEG.times(1,end)
        error('Exceeds limits and overlaps with TMS data!');
    elseif -EEG.times(imin2)+sbeforerec2 < EEG.times(1,EEG.TMS_period2remove_1(1,end)) || EEG.times(imin1)+safterrec1> EEG.times(1,EEG.TMS_period2remove_b(1,1))
        error('Exceeds limits and overlaps with TMS data!');
    end
	%extra baseline correction
	if cor == 1
		dur = EEG.pnts/4;
		dur = dur*EEG.srate/1000;
		npreTMS = min([sbeforerec1, sbeforerec2, safterrec1, safterrec2, dur]);
		npostTMS = npreTMS;
		EEG = baselineCorForGaps(EEG,npreTMS,npostTMS,sbeforeTMS1,safterTMS1,imin1);
		EEG = baselineCorForGaps(EEG,npreTMS,npostTMS,sbeforeTMS2,safterTMS2,imin2);
	end
    %fill gaps
    EEG = fillGaps(EEG,sbeforerec1,safterrec1,sberofeTMS1,safterTMS1,imin1,memb,nnei);
    EEG = fillGaps(EEG,sbeforerec2,safterrec2,sberofeTMS2,safterTMS2,imin2,memb,nnei);
	%add new indexes and information in tmscut
    EEG.tmscut(z+1).cutTimesTMS = EEG.tmscut(z).cutTimesTMS;
    EEG.tmscut(z+1).replaceTimes = EEG.tmscut(z+1).cutTimesTMS;
    EEG.tmscut(z+1).replacement = [{EEG.data(:,sbeforeTMS1:safterTMS1,:)},{EEG.data(:,sbeforeTMS2:safterTMS2,:)}];
    EEG.tmscut(z+1).tmscut1 = sbeforeTMS1;
    EEG.tmscut(z+1).tmscut2 = safterTMS1;
    EEG.tmscut(z+1).tmscut3 = sbeforeTMS2;
    EEG.tmscut(z+1).tmscut3 = safterTMS2;
    EEG.tmscut(z+1).srate = EEG.srate;
    EEG.tmscut(z+1).interpolated = 'applied prediction';    
end
end

